﻿using System.IO;
using System.Linq;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using System.Web.Routing;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for FormTest and is intended
	///to contain all FormTest Unit Tests
	///</summary>
	[TestClass()]
	public class AjaxFormTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;

		private const string onClickScript = @"onclick=""Sys.Mvc.AsyncForm.handleClick(this, new Sys.UI.DomEvent(event));""";
		private const string onSubmitScript = @"onsubmit=""Sys.Mvc.AsyncForm.handleSubmit(this, new Sys.UI.DomEvent(event), { insertionMode: Sys.Mvc.InsertionMode.replace });""";
		private const string onSubmitScriptWithTargetId = @"onsubmit=""Sys.Mvc.AsyncForm.handleSubmit(this, new Sys.UI.DomEvent(event), { insertionMode: Sys.Mvc.InsertionMode.replace, updateTargetId: &#39;targetId&#39; });""";
		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			if (!RouteTable.Routes.Any()) {
				RouteTable.Routes.MapRoute(
					"Default", // Route name
					"{controller}/{action}/{id}", // URL with parameters
					new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
				);
			}
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}

		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
			view.ViewContext.Writer.Close();
			view.ViewContext.Writer = new StringWriter();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}

		#endregion

		/// <summary>
		///A test for Form Constructor
		///</summary>
		[TestMethod()]
		public void AjaxForm_NoParameters() {
			// Arrange
			var outputUrl = "/Home/Index.aspx";

			// Act
			var form = view.Ajax.Form();
			var expectedOutput = string.Format(@"<form action=""{0}"" id=""{1}"" method=""post"" {2} {3}>", outputUrl, form.Id, onClickScript, onSubmitScript);
			var output = view.ViewContext.Writer.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Form Constructor
		///</summary>
		[TestMethod()]
		public void AjaxForm_WithFormAction() {
			// Arrange
			var formAction = "~/link/dummy.aspx";
			var outputUrl = "/link/dummy.aspx";

			// Act
			var form = view.Ajax.Form(formAction);
			var expectedOutput = string.Format(@"<form action=""{0}"" id=""{1}"" method=""post"" {2} {3}>", outputUrl, form.Id, onClickScript, onSubmitScript);
			var output = view.ViewContext.Writer.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Form Constructor
		///</summary>
		[TestMethod()]
		public void AjaxForm_WithAjaxOptions() {
			// Arrange
			var outputUrl = "/Home/Index.aspx";
			var ajaxOptions = new AjaxOptions() {
				UpdateTargetId = "targetId"
			};

			// Act
			var form = view.Ajax.Form(ajaxOptions: ajaxOptions);
			var expectedOutput = string.Format(@"<form action=""{0}"" id=""{1}"" method=""post"" {2} {3}>", outputUrl, form.Id, onClickScript, onSubmitScriptWithTargetId);
			var output = view.ViewContext.Writer.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Form Constructor
		///</summary>
		[TestMethod()]
		public void AjaxActionForm_NoParameters() {
			// Arrange
			var outputUrl = "/";

			// Act
			var form = view.Ajax.ActionForm();
			var expectedOutput = string.Format(@"<form action=""{0}"" id=""{1}"" method=""post"" {2} {3}>", outputUrl, form.Id, onClickScript, onSubmitScript);
			var output = view.ViewContext.Writer.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Form Constructor
		///</summary>
		[TestMethod()]
		public void AjaxRouteForm_NoParameters() {
			// Arrange
			var outputUrl = "/";

			// Act
			var form = view.Ajax.RouteForm();
			var expectedOutput = string.Format(@"<form action=""{0}"" id=""{1}"" method=""post"" {2} {3}>", outputUrl, form.Id, onClickScript, onSubmitScript);
			var output = view.ViewContext.Writer.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
