﻿using System.Linq;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for DisplayTextTest and is intended
	///to contain all DisplayTextTest Unit Tests
	///</summary>
	[TestClass()]
	public class DisplayTextTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;
		private static ModelViewForm<Dummy> form;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			form = new ModelViewForm<Dummy>(view.Html);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for DisplayText Constructor
		///</summary>
		[TestMethod()]
		public void DisplayText_WithPropertyName() {
			// Arrange
			var name = "BasicProperty";
			var dummy = database.Dummies.All().First();
			var expectedOutput = string.Format(@"<span id=""{1}_Text"" type=""displayText"">{0}</span><input id=""{1}"" type=""hidden"" value=""{0}"" />", dummy.BasicProperty, name);

			// Act
			var staticText = form.DisplayText(name);
			var output = staticText.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for DisplayText Constructor
		///</summary>
		[TestMethod()]
		public void DisplayText_ForModel() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var expectedName = "BasicProperty";
			var expectedOutput = string.Format(@"<span id=""{1}_Text"" type=""displayText"">{0}</span><input id=""{1}"" type=""hidden"" value=""{0}"" />", expectedValue, expectedName);

			// Act
			var staticText = form.DisplayText(m => m.BasicProperty);
			var output = staticText.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for DisplayText Constructor
		///</summary>
		[TestMethod()]
		public void DisplayText_ForEmptyProperty() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.EmptyProperty;
			var expectedName = "EmptyProperty";
			var expectedOutput = string.Format(@"<span id=""{1}_Text"" type=""displayText""></span><input id=""{1}"" type=""hidden"" value=""{0}"" />", expectedValue, expectedName);

			// Act
			var staticText = form.DisplayText(m => m.EmptyProperty);
			var output = staticText.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for DisplayText Constructor
		///</summary>
		[TestMethod()]
		public void DisplayText_WithNullDisplayText() {
			// Arrange
			var name = "EmptyProperty";
			var nullValue = "This is Empty";
			var expectedOutput = string.Format(@"<span id=""{1}_Text"" type=""displayText"">{0}</span><input id=""{1}"" type=""hidden"" value="""" />", nullValue, name);

			// Act
			var staticText = form.DisplayText(name).NullDisplayValue(nullValue);
			var output = staticText.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for DisplayText Constructor
		///</summary>
		[TestMethod()]
		public void DisplayText_WithAutoLabel() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.LabeledProperty;
			var expectedName = "LabeledProperty";
			var expectedLabel = "Labeled Property";
			var expectedOutput = string.Format(@"<label for=""{1}_Text"" id=""{1}_Text_Label"">{2}:&nbsp;</label><span id=""{1}_Text"" type=""displayText"">{0}</span><input id=""{1}"" type=""hidden"" value=""{0}"" />", expectedValue, expectedName, expectedLabel);

			// Act
			var staticText = form.DisplayText(m => m.LabeledProperty).AddLabel();
			var output = staticText.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
