﻿using System.Linq;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {

	/// <summary>
	///This is a test class for LabelTest and is intended
	///to contain all LabelTest Unit Tests
	///</summary>
	[TestClass()]
	public class LabelTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;
		private static ModelViewForm<Dummy> form;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			form = new ModelViewForm<Dummy>(view.Html);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for Label Constructor
		///</summary>
		[TestMethod()]
		public void Label_ForBasicProperty() {
			// Arrange
			var memberName = "BasicProperty";
			var expectedOutput = string.Format(@"<label for=""{0}"" id=""{0}_Label"">{0}</label>", memberName);

			// Act
			var label = form.LabelFor(memberName);
			var output = label.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Label Constructor
		///</summary>
		[TestMethod()]
		public void Label_ForModel() {
			// Arrange
			var memberName = "LabeledProperty";
			var expectedLabel = "Labeled Property";
			var expectedOutput = string.Format(@"<label for=""{0}"" id=""{0}_Label"">{1}</label>", memberName, expectedLabel);

			// Act
			var label = form.LabelFor(m => m.LabeledProperty);
			var output = label.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Label Constructor
		///</summary>
		[TestMethod()]
		public void Label_ForBasicProperty_WithDiferentId() {
			// Arrange
			var controlId = "ControlId";
			var memberName = "BasicProperty";
			var expectedOutput = string.Format(@"<label for=""{0}"" id=""{0}_Label"">{1}</label>", controlId, memberName);

			// Act
			var label = form.LabelFor(memberName, controlId);
			var output = label.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Label Constructor
		///</summary>
		[TestMethod()]
		public void Label_ForRequiredProperty() {
			// Arrange
			var memberName = "RequiredProperty";
			var expectedOutput = string.Format(@"<label for=""{0}"" id=""{0}_Label"" required=""required"">{0}</label>", memberName);

			// Act
			var label = form.LabelFor(memberName);
			var output = label.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Label Constructor
		///</summary>
		[TestMethod()]
		public void Label_ForLabeledProperty() {
			// Arrange
			var memberName = "LabeledProperty";
			var expectedLabel = "Labeled Property";
			var expectedOutput = string.Format(@"<label for=""{0}"" id=""{0}_Label"">{1}</label>", memberName, expectedLabel);

			// Act
			var label = form.LabelFor(memberName);
			var output = label.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Label Constructor
		///</summary>
		[TestMethod()]
		public void Label_ForLabeledProperty_WithControlId() {
			// Arrange
			var controlId = "ControlId";
			var memberName = "LabeledProperty";
			var expectedLabel = "Labeled Property";
			var expectedOutput = string.Format(@"<label for=""{0}"" id=""{0}_Label"">{1}</label>", controlId, expectedLabel);

			// Act
			var label = form.LabelFor(memberName, controlId);
			var output = label.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
