﻿using System.Linq;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for PasswordTest and is intended
	///to contain all PasswordTest Unit Tests
	///</summary>
	[TestClass()]
	public class PasswordTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;
		private static ModelViewForm<Dummy> form;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			form = new ModelViewForm<Dummy>(view.Html);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for Password Constructor
		///</summary>
		[TestMethod()]
		public void Password_WithPropertyName() {
			// Arrange
			var name = "BasicProperty";
			var dummy = database.Dummies.All().First();
			var expectedOutput = string.Format(@"<input id=""{1}"" name=""{1}"" type=""password"" />", dummy.BasicProperty, name);

			// Act
			var password = form.Password(name);
			var output = password.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Password Constructor
		///</summary>
		[TestMethod()]
		public void Password_ForModel() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var expectedName = "BasicProperty";
			var expectedOutput = string.Format(@"<input id=""{1}"" name=""{1}"" type=""password"" />", expectedValue, expectedName);

			// Act
			var password = form.Password(m => m.BasicProperty);
			var output = password.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Password Constructor
		///</summary>
		[TestMethod()]
		public void Password_ForRequiredField() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.RequiredProperty;
			var expectedName = "RequiredProperty";
			var expectedOutput = string.Format(@"<input id=""{1}"" name=""{1}"" required=""required"" type=""password"" />", expectedValue, expectedName);

			// Act
			var password = form.Password(m => m.RequiredProperty);
			var output = password.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Password Constructor
		///</summary>
		[TestMethod()]
		public void Password_ForRequiredField_WithValidationError() {
			// Arrange
			view.ViewContext.ViewData.ModelState.AddModelError("RequiredProperty", "Field is required.");
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.RequiredProperty;
			var expectedName = "RequiredProperty";
			var expectedOutput = string.Format(@"<input class=""input-validation-error"" id=""{1}"" name=""{1}"" required=""required"" type=""password"" />", expectedValue, expectedName);

			// Act
			var password = form.Password(m => m.RequiredProperty);
			var output = password.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Password Constructor
		///</summary>
		[TestMethod()]
		public void Password_ForRequiredField_WithValidationMessage() {
			// Arrange
			var errrorMessage = "Field is required.";
			view.ViewContext.ViewData.ModelState.AddModelError("RequiredProperty", errrorMessage);
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.RequiredProperty;
			var expectedName = "RequiredProperty";
			var expectedOutput = string.Format(@"<input class=""input-validation-error"" id=""{1}"" name=""{1}"" required=""required"" type=""password"" /><span class=""field-validation-error"" id=""RequiredProperty_Error"">{2}</span>", expectedValue, expectedName, errrorMessage);

			// Act
			var password = form.Password(m => m.RequiredProperty).AddValidationMessage();
			var output = password.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Password Constructor
		///</summary>
		[TestMethod()]
		public void Password_WithMaxLength() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.SizedProperty;
			var expectedName = "SizedProperty";
			var expectedOutput = string.Format(@"<input id=""{1}"" maxlength=""32"" name=""{1}"" type=""password"" />", expectedValue, expectedName);

			// Act
			var password = form.Password(m => m.SizedProperty).MaxLength(32);
			var output = password.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for Password Constructor
		///</summary>
		[TestMethod()]
		public void Password_WithSize() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.SizedProperty;
			var expectedName = "SizedProperty";
			var expectedOutput = string.Format(@"<input id=""{1}"" name=""{1}"" size=""32"" type=""password"" />", expectedValue, expectedName);

			// Act
			var password = form.Password(m => m.SizedProperty).Size(32);
			var output = password.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
