﻿using System.Linq;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for ValidationMessageTest and is intended
	///to contain all ValidationMessageTest Unit Tests
	///</summary>
	[TestClass()]
	public class ValidationMessageTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;
		private static ModelViewForm<Dummy> form;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			form = new ModelViewForm<Dummy>(view.Html);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for ValidationMessage Constructor
		///</summary>
		[TestMethod()]
		public void ValidationMessage_WithNoError() {
			// Arrange
			var name = "BasicProperty";
			var expectedOutput = string.Format(@"");

			// Act
			var validationMessage = form.ValidationMessageFor(name);
			var output = validationMessage.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
			Assert.IsFalse(validationMessage.HasError);
			Assert.AreEqual<string>(null, validationMessage.ErrorMessage);
		}

		/// <summary>
		///A test for ValidationMessage Constructor
		///</summary>
		[TestMethod()]
		public void ValidationMessage_WithPropertyName() {
			// Arrange
			var errrorMessage = "Model error message.";
			view.ViewContext.ViewData.ModelState.AddModelError("BasicProperty", errrorMessage);
			var name = "BasicProperty";
			var expectedOutput = string.Format(@"<span class=""field-validation-error"" id=""{0}_Error"">{1}</span>", name, errrorMessage);

			// Act
			var validationMessage = form.ValidationMessageFor(name);
			var output = validationMessage.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
			Assert.IsTrue(validationMessage.HasError);
			Assert.AreEqual<string>(errrorMessage, validationMessage.ErrorMessage);
		}

		/// <summary>
		///A test for ValidationMessage Constructor
		///</summary>
		[TestMethod()]
		public void ValidationMessage_ForModel() {
			// Arrange
			var errrorMessage = "Model error message.";
			view.ViewContext.ViewData.ModelState.AddModelError("BasicProperty", errrorMessage);
			var expectedName = "BasicProperty";
			var expectedOutput = string.Format(@"<span class=""field-validation-error"" id=""{0}_Error"">{1}</span>", expectedName, errrorMessage);

			// Act
			var validationMessage = form.ValidationMessageFor(m => m.BasicProperty);
			var output = validationMessage.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ValidationMessage Constructor
		///</summary>
		[TestMethod()]
		public void ValidationMessage_ForModel_WithCustomMessage() {
			// Arrange
			var errrorMessage = "Model error message.";
			view.ViewContext.ViewData.ModelState.AddModelError("BasicProperty", errrorMessage);
			var expectedName = "BasicProperty";
			var informedMessage = "This is a custom message.";
			var expectedOutput = string.Format(@"<span class=""field-validation-error"" id=""{0}_Error"">{1}</span>", expectedName, informedMessage);

			// Act
			var validationMessage = form.ValidationMessageFor(m => m.BasicProperty).DefaultMessage(informedMessage);
			var output = validationMessage.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
			Assert.IsTrue(validationMessage.HasError);
			Assert.AreEqual<string>(informedMessage, validationMessage.ErrorMessage);
		}
	}
}
