﻿using System.Linq;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for ValidationSummaryTest and is intended
	///to contain all ValidationSummaryTest Unit Tests
	///</summary>
	[TestClass()]
	public class ValidationSummaryTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;
		private static ModelViewForm<Dummy> form;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			form = new ModelViewForm<Dummy>(view.Html);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for ValidationSummary Constructor
		///</summary>
		[TestMethod()]
		public void ValidationSummary_WithNoErrors() {
			// Arrange
			var expectedOutput = string.Format(@"");

			// Act
			var validationSummary = form.ValidationSummary();
			var output = validationSummary.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ValidationSummary Constructor
		///</summary>
		[TestMethod()]
		public void ValidationSummary_NoParameters() {
			// Arrange
			var modelErrorMessage = "Model error message.";
			var propertyErrorMessage = "Basic property error message.";
			view.ViewContext.ViewData.ModelState.AddModelError("", modelErrorMessage);
			view.ViewContext.ViewData.ModelState.AddModelError("BasicProperty", propertyErrorMessage);
			var expectedOutput = string.Format(@"<div class=""validation-summary-errors""><ul><li>{0}</li><li>{1}</li></ul></div>", modelErrorMessage, propertyErrorMessage);

			// Act
			var validationSummary = form.ValidationSummary();
			var output = validationSummary.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ValidationSummary Constructor
		///</summary>
		[TestMethod()]
		public void ValidationSummary_WithCustomMessage() {
			// Arrange
			var modelErrorMessage = "Model error message.";
			var propertyErrorMessage = "Basic property error message.";
			view.ViewContext.ViewData.ModelState.AddModelError("", modelErrorMessage);
			view.ViewContext.ViewData.ModelState.AddModelError("BasicProperty", propertyErrorMessage);
			var informedMessage = "This is a custom message.";
			var expectedOutput = string.Format(@"<div class=""validation-summary-errors""><span>{2}</span><ul><li>{0}</li><li>{1}</li></ul></div>", modelErrorMessage, propertyErrorMessage, informedMessage);

			// Act
			var validationSummary = form.ValidationSummary(informedMessage);
			var output = validationSummary.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ValidationSummary Constructor
		///</summary>
		[TestMethod()]
		public void ValidationSummary_ExcludingProperties() {
			// Arrange
			var modelErrorMessage = "Model error message.";
			var propertyErrorMessage = "Basic property error message.";
			view.ViewContext.ViewData.ModelState.AddModelError("", modelErrorMessage);
			view.ViewContext.ViewData.ModelState.AddModelError("BasicProperty", propertyErrorMessage);
			var expectedOutput = string.Format(@"<div class=""validation-summary-errors""><ul><li>{0}</li></ul></div>", modelErrorMessage, propertyErrorMessage);

			// Act
			var validationSummary = form.ValidationSummary().ExcludePropertyErrors();
			var output = validationSummary.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
