﻿using System.Linq;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for ListItemTest and is intended
	///to contain all ListItemTest Unit Tests
	///</summary>
	[TestClass()]
	public class ListItemTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for ListItem Constructor
		///</summary>
		[TestMethod()]
		public void ListItem_NoParameters() {
			// Arrange
			var expectedOutput = string.Format(@"<li></li>");

			// Act
			var listItem = view.Html.ListItem();
			var output = listItem.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListItem Constructor
		///</summary>
		[TestMethod()]
		public void ListItem_WithItems() {
			// Arrange
			var content = "Item1";
			var expectedOutput = string.Format(@"<li>{0}</li>", content);

			// Act
			var listItem = view.Html.ListItem(content);
			var output = listItem.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
