using System.Web.Mvc;
using ExtendedMvc.Ajax;
using ExtendedMvc.Html;

namespace ExtendedMvc.Elements {
	public class ResetImage<TModel> : ImageBase<ResetImage<TModel>> where TModel : class {
		private string _formId;
		private bool _disabled;
		private string _disabledSource;
		private IModelViewHelper<TModel> _helper;

		public ResetImage(IModelViewHelper<TModel> helper, string source, string formId)
			: base(source, null) {
			SetAttribute(HtmlAttribute.Type, "reset");
			_helper = helper;
			_formId = formId;
		}

		public virtual ResetImage<TModel> Disabled(bool value = true) {
			_disabled = value;
			return this;
		}

		public virtual ResetImage<TModel> DisabledImage(string disabledSource = null) {
			_disabledSource = disabledSource;
			return this;
		}

		protected override void Prepare() {
			OnClick(_disabled ? "javascript: return false;" : string.Format(JavaScriptHelper.Reset, _formId));
			if (_disabled) {
				SetAttribute(HtmlAttribute.Disabled, HtmlAttribute.Disabled);
				if (_disabledSource != null) Source(_disabledSource);
			}
			Source(UrlHelper.GenerateContentUrl(source, _helper.ViewContext.HttpContext));
			base.Prepare();
		}
	}
}