using System.Web.Mvc;
using System.Web.Routing;

namespace ExtendedMvc {
	/// <summary>
	/// Extensions to ModelViewHelper
	/// </summary>
	public static class HtmlFormExtensions {
		public static ModelViewForm<T> Form<T>(this ModelViewHtmlHelper<T> html, string formAction = null, FormMethod method = FormMethod.Post) where T : class {
			var link = html.ViewContext.HttpContext.Request.RawUrl;
			if (formAction != null) link = UrlHelper.GenerateContentUrl(formAction, html.ViewContext.HttpContext);
			return ModelViewForm<T>.Create(html, link, method);
		}

		public static ModelViewForm<T> ActionForm<T>(this ModelViewHtmlHelper<T> html, string actionName = null, string controllerName = null, object routeValues = null, FormMethod method = FormMethod.Post) where T : class {
			var values = routeValues is RouteValueDictionary ? routeValues as RouteValueDictionary : new RouteValueDictionary(routeValues);
			var formAction = UrlHelper.GenerateUrl(null, actionName, controllerName, values, html.RouteCollection, html.ViewContext.RequestContext, true);
			return html.Form(formAction, method);
		}

		public static ModelViewForm<T> RouteForm<T>(this ModelViewHtmlHelper<T> html, string routeName = null, object routeValues = null, FormMethod method = FormMethod.Post) where T : class {
			var values = routeValues is RouteValueDictionary ? routeValues as RouteValueDictionary : new RouteValueDictionary(routeValues);
			string formAction = UrlHelper.GenerateUrl(routeName, null, null, values, html.RouteCollection, html.ViewContext.RequestContext, false);
			return html.Form(formAction, method);
		}
	}
}
