using System.Web.Mvc;
using System.Web.Routing;
using ExtendedMvc.Elements;

namespace ExtendedMvc {
	/// <summary>
	/// Extensions to ModelViewHelper
	/// </summary>
	public static class HtmlLinkExtensions {

		public static Link Link<T>(this ModelViewHtmlHelper<T> html, string content, string link) where T : class {
			link = UrlHelper.GenerateContentUrl(link, html.ViewContext.HttpContext);
			return new Link(content, link);
		}

		public static Link ActionLink<T>(this ModelViewHtmlHelper<T> html, string content, string actionName = null, string controllerName = null, object routeValues = null) where T : class {
			var values = routeValues is RouteValueDictionary ? routeValues as RouteValueDictionary : new RouteValueDictionary(routeValues);
			var link = UrlHelper.GenerateUrl(null, actionName, controllerName, values, html.RouteCollection, html.ViewContext.RequestContext, true);
			return html.Link(content, link);
		}

		public static Link RouteLink<T>(this ModelViewHtmlHelper<T> html, string content, string routeName, string protocol = null, string hostName = null, string fragment = null, object routeValues = null) where T : class {
			var values = routeValues is RouteValueDictionary ? routeValues as RouteValueDictionary : new RouteValueDictionary(routeValues);
			var link = UrlHelper.GenerateUrl(routeName, null, null, protocol, hostName, fragment, values, html.RouteCollection, html.ViewContext.RequestContext, true);
			return html.Link(content, link);
		}

		public static ImageLink ImageLink<T>(this ModelViewHtmlHelper<T> html, string source, string link) where T : class {
			source = UrlHelper.GenerateContentUrl(source, html.ViewContext.HttpContext);
			link = UrlHelper.GenerateContentUrl(link, html.ViewContext.HttpContext);
			return new ImageLink(source, link);
		}

		public static ImageLink ActionImageLink<T>(this ModelViewHtmlHelper<T> html, string source, string actionName = null, string controllerName = null, object routeValues = null) where T : class {
			var values = routeValues is RouteValueDictionary ? routeValues as RouteValueDictionary : new RouteValueDictionary(routeValues);
			var link = UrlHelper.GenerateUrl(null, actionName, controllerName, values, html.RouteCollection, html.ViewContext.RequestContext, true);
			return html.ImageLink(source, link);
		}

		public static ImageLink RouteImageLink<T>(this ModelViewHtmlHelper<T> html, string source, string routeName = null, string protocol = null, string hostName = null, string fragment = null, object routeValues = null) where T : class {
			var values = routeValues is RouteValueDictionary ? routeValues as RouteValueDictionary : new RouteValueDictionary(routeValues);
			var link = UrlHelper.GenerateUrl(routeName, null, null, protocol, hostName, fragment, values, html.RouteCollection, html.ViewContext.RequestContext, true);
			return html.ImageLink(source, link);
		}
	}
}
