﻿using System.Linq;
using System.Web.Mvc;
using System.Web.Routing;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for ImageLinkTest and is intended
	///to contain all ImageLinkTest Unit Tests
	///</summary>
	[TestClass()]
	public class AjaxImageLinkTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;

		private const string onClickScript = @"onclick=""Sys.Mvc.AsyncHyperlink.handleClick(this, new Sys.UI.DomEvent(event), { insertionMode: Sys.Mvc.InsertionMode.replace });""";

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			if (!RouteTable.Routes.Any()) {
				RouteTable.Routes.MapRoute(
					"Default", // Route name
					"{controller}/{action}/{id}", // URL with parameters
					new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
				);
			}
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for ImageLink Constructor
		///</summary>
		[TestMethod()]
		public void AjaxImageLink_WithUrl() {
			// Arrange
			var linkImageSource = "~/images/dummy.jpg";
			var imageSourceOutput = "/images/dummy.jpg";
			var imageLinkUrl = "~/imageLink/dummy.aspx";
			var outputUrl = "/imageLink/dummy.aspx";
			var expectedOutput = string.Format(@"<a href=""{1}"" {2}><image src=""{0}"" /></a>", imageSourceOutput, outputUrl, onClickScript);

			// Act
			var imageLink = view.Ajax.ImageLink(linkImageSource, imageLinkUrl);
			var output = imageLink.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ImageLink Constructor
		///</summary>
		[TestMethod()]
		public void AjaxActionImageLink_WithUrl() {
			// Arrange
			var linkImageSource = "~/images/dummy.jpg";
			var imageSourceOutput = "/images/dummy.jpg";
			var outputUrl = "/";
			var expectedOutput = string.Format(@"<a href=""{1}"" {2}><image src=""{0}"" /></a>", imageSourceOutput, outputUrl, onClickScript);

			// Act
			var imageLink = view.Ajax.ActionImageLink(linkImageSource);
			var output = imageLink.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ImageLink Constructor
		///</summary>
		[TestMethod()]
		public void AjaxRouteImageLink_WithUrl() {
			// Arrange
			var linkImageSource = "~/images/dummy.jpg";
			var imageSourceOutput = "/images/dummy.jpg";
			var outputUrl = "/";
			var expectedOutput = string.Format(@"<a href=""{1}"" {2}><image src=""{0}"" /></a>", imageSourceOutput, outputUrl, onClickScript);

			// Act
			var imageLink = view.Ajax.RouteImageLink(linkImageSource);
			var output = imageLink.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
