﻿using System.Linq;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for CheckBoxTest and is intended
	///to contain all CheckBoxTest Unit Tests
	///</summary>
	[TestClass()]
	public class CheckBoxTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;
		private static ModelViewForm<Dummy> form;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testContext) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			form = new ModelViewForm<Dummy>(view.Html);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for CheckBox Constructor
		///</summary>
		[TestMethod()]
		public void CheckBox_WithPropertyName_Unchecked() {
			// Arrange
			var name = "BoolProperty";
			var dummy = database.Dummies.All().First();
			dummy.BoolProperty = false;
			var expectedOutput = string.Format(@"<input id=""{0}"" name=""{0}"" required=""required"" type=""checkbox"" value=""False"" />", name);

			// Act
			var checkBox = form.SingleCheckBox(name);
			var output = checkBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for CheckBox Constructor
		///</summary>
		[TestMethod()]
		public void CheckBox_WithPropertyName_Checked() {
			// Arrange
			var name = "BoolProperty";
			var dummy = database.Dummies.All().First();
			dummy.BoolProperty = true;
			var expectedOutput = string.Format(@"<input checked=""checked"" id=""{0}"" name=""{0}"" required=""required"" type=""checkbox"" value=""True"" />", name);

			// Act
			var checkBox = form.SingleCheckBox(name);
			var output = checkBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for CheckBox Constructor
		///</summary>
		[TestMethod()]
		public void CheckBox_ForModel() {
			// Arrange
			var expectedName = "BoolProperty";
			var dummy = database.Dummies.All().First();
			dummy.BoolProperty = false;
			var expectedOutput = string.Format(@"<input id=""{0}"" name=""{0}"" required=""required"" type=""checkbox"" value=""False"" />", expectedName);

			// Act
			var checkBox = form.SingleCheckBox(m => m.BoolProperty);
			var output = checkBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
