﻿using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using ExtendedMvc.Tests.Model;
using ExtendedMvc.Tests.Model.Entities;
using ExtendedMvc.Tests.Model.Enumerations;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ExtendedMvc.Tests {


	/// <summary>
	///This is a test class for ListBoxTest and is intended
	///to contain all ListBoxTest Unit Tests
	///</summary>
	[TestClass()]
	public class ListBoxTest {

		private static DummyDataSource database;
		private static ModelViewPage<Dummy> view;
		private static ModelViewForm<Dummy> form;

		/// <summary>
		///Gets or sets the test conradio which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		[ClassInitialize()]
		public static void MyClassInitialize(TestContext testConradio) {
			database = new DummyDataSource();
			var dummy = database.Dummies.All().First();
			view = MvcMockHelpers.FakeViewPage("Index", dummy);
			form = new ModelViewForm<Dummy>(view.Html);
		}

		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup() {
		//}
		//
		//Use TestInitialize to run code before running each test
		[TestInitialize()]
		public void MyTestInitialize() {
			view.ViewContext.ViewData.ModelState.Clear();
		}

		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup() {
		//}
		//
		#endregion

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		[ExpectedException(typeof(System.ArgumentException))]
		public void ListBox_WithPropertyName_WithNoOption() {
			// Arrange
			var name = "BasicProperty";

			// Act
			var listBox = form.ListBox(name);
			var output = listBox.ToString();
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_WithPropertyName_WithOptionArray() {
			// Arrange
			var name = "BasicProperty";
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var options = new object[] { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", name, options[0], options[1], expectedValue);

			// Act
			var listBox = form.ListBox(name).Options(options);
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_WithPropertyName_WithSingleSelection() {
			// Arrange
			var name = "BasicProperty";
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var options = new object[] { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"">{1}</option>" +
@"<option id=""{0}_{2}"" selected=""selected"">{2}</option>" +
@"</select>", name, options[0], options[1], expectedValue);

			// Act
			var listBox = form.ListBox(name).Options(options).Select("OtherValue");
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_WithPropertyName_WithSelectFromList() {
			// Arrange
			var name = "BasicProperty";
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var options = new object[] { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"">{1}</option>" +
@"<option id=""{0}_{2}"" selected=""selected"">{2}</option>" +
@"</select>", name, options[0], options[1], expectedValue);

			// Act
			var listBox = form.ListBox(name).Options(options).Select(new List<object>() { "OtherValue" });
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_WithPropertyName_WithSelectFromDictionary() {
			// Arrange
			var name = "BasicProperty";
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var options = new object[] { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"">{1}</option>" +
@"<option id=""{0}_{2}"" selected=""selected"">{2}</option>" +
@"</select>", name, options[0], options[1], expectedValue);

			// Act
			var listBox = form.ListBox(name).Options(options).Select(new Dictionary<object, object>() { { "OtherValue", "OtherValue" } });
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForModel_WithOptionArray() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var expectedName = "BasicProperty";
			var options = new object[] { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", expectedName, options[0], options[1]);

			// Act
			var listBox = form.ListBox(expectedName).Options(options);
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForModel_WithMultiSelectList() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var expectedName = "BasicProperty";
			var options = new SelectListItem[] { new SelectListItem() { Value = expectedValue, Text = "Text1" }, new SelectListItem() { Value = "OtherValue", Text = "OtherValue" } };
			var multiSelectList = new MultiSelectList(options, "Value", "Text");
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", expectedName, options[0].Value, options[1].Value, options[0].Text, options[1].Text);

			// Act
			var listBox = form.ListBox(expectedName).Options(multiSelectList);
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForModel_WithIEnumerableItem() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var expectedName = "BasicProperty";
			var options = new List<string> { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", expectedName, options[0], options[1], options[0], options[1]);

			// Act
			var listBox = form.ListBox(expectedName).Options(options);
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForModel_WithIDictionary() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var expectedName = "BasicProperty";
			var options = new Dictionary<object, string>();
			options.Add((object)expectedValue, "Text1");
			options.Add((object)"OtherValue", "Text2");
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", expectedName, expectedValue, "OtherValue", options[expectedValue], options["OtherValue"]);

			// Act
			var listBox = form.ListBox(expectedName).Options(options);
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForModel_WithAnonimousEnumerable() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.BasicProperty;
			var expectedName = "BasicProperty";
			var options = new object[] { new { Value = expectedValue, Text = "Text1" }, new { Value = "OtherValue", Text = "Text2" } };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", expectedName, expectedValue, "OtherValue", "Text1", "Text2");

			// Act
			var listBox = form.ListBox(expectedName).Options(options, "Value", "Text");
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForModel_WithEnum() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.EnumProperty;
			var expectedName = "EnumProperty";
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", expectedName, DummyEnum.FirstValue.ToString(), DummyEnum.SecondValue.ToString(), DummyEnum.FirstValue.ToString(), DummyEnum.SecondValue.ToString());

			// Act
			var listBox = form.ListBox(expectedName).Options<DummyEnum>();
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForRequiredProperty() {
			// Arrange
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.RequiredProperty;
			var expectedName = "RequiredProperty";
			var options = new object[] { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select id=""{0}"" multiple=""multiple"" name=""{0}"" required=""required"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>", expectedName, options[0], options[1]);

			// Act
			var listBox = form.ListBox(expectedName).Options(options);
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}

		/// <summary>
		///A test for ListBox Constructor
		///</summary>
		[TestMethod()]
		public void ListBox_ForRequiredField_WithValidationMessage() {
			// Arrange
			var expectedErrorMessage = "Field is required.";
			view.ViewContext.ViewData.ModelState.AddModelError("RequiredProperty", expectedErrorMessage);
			var dummy = database.Dummies.All().First();
			var expectedValue = dummy.RequiredProperty;
			var expectedName = "RequiredProperty";
			var options = new object[] { expectedValue, "OtherValue" };
			var expectedOutput = string.Format(
@"<select class=""input-validation-error"" id=""{0}"" multiple=""multiple"" name=""{0}"" required=""required"">" +
@"<option id=""{0}_{1}"" selected=""selected"">{1}</option>" +
@"<option id=""{0}_{2}"">{2}</option>" +
@"</select>" +
@"<span class=""field-validation-error"" id=""{0}_Error"">{3}</span>"
, expectedName, options[0], options[1], expectedErrorMessage);

			// Act
			var listBox = form.ListBox(expectedName).Options(options).AddValidationMessage();
			var output = listBox.ToString();

			// Assert
			Assert.AreEqual<string>(expectedOutput, output);
		}
	}
}
