using ExtendedMvc.Common.Diagnostics;
using ExtendedMvc.Html;

namespace ExtendedMvc.Elements {
	public class File<TModel> : FileBase<File<TModel>, TModel> where TModel : class {
		public File(IModelViewHelper<TModel> helper, string memberName, object value, string id = null) : base(helper, HtmlInputType.File, memberName, value, id) { }
	}

	public abstract class FileBase<T, TModel> : Input<T, TModel>
		where T : FileBase<T, TModel>
		where TModel : class {
		private int? _maxLength;
		private int? _size;

		protected FileBase(IModelViewHelper<TModel> helper, string type, string memberName, object value, string id = null) : base(helper, type, value, memberName, id) { }

		public virtual T MaxLength(int value) {
			Check.Argument.IsNotNegative(value, "value");
			_maxLength = value == 0 ? (int?)null : value;
			return (T)this;
		}

		public virtual T Size(int value) {
			Check.Argument.IsNotNegative(value, "value");
			_size = value == 0 ? (int?)null : value;
			return (T)this;
		}

		protected override void Prepare() {
			if (_maxLength != null) SetAttribute(HtmlAttribute.MaxLength, _maxLength.Value);
			if (_size != null) SetAttribute(HtmlAttribute.Size, _size.Value);
			base.Prepare();
		}
	}
}
